@@ -1,12 +1,12 @@
 ### import, check metabolomic data
 ### or other continuous omic data
 
-metabolomicdata <- function(file, check = TRUE)
+metabolomicdata <- function(file, backgrounddose, check = TRUE)
 {
-  continuousomicdata(file, check = check)
+  continuousomicdata(file, backgrounddose, check = check)
 }
   
-continuousomicdata <- function(file, check = TRUE)
+continuousomicdata <- function(file, backgrounddose, check = TRUE)
 {
   if (is.data.frame(file))
   {
@@ -75,7 +75,18 @@ continuousomicdata <- function(file, check = TRUE)
   row.names(data) <- item <- as.character(d[2:nrowd, 1])
   (nitems <- nrow(data))
   
+  if (!missing(backgrounddose))
+  {
+    dose <- dose * (dose > backgrounddose)
+  }
+  
   # control of the design
+  if (!any(dose == 0))
+    stop(strwrap(prefix = "\n", initial = "\n",
+                 "DRomics cannot be used on a design with no dose at zero. 
+            In case of observational data, to prevent calculation of BMDs by extrapolation, 
+            doses considered as corresponding to the background exposition (control) must 
+                 be fixed at 0. You can use the argument backgrounddose for that purpose."))
   if (any(dose < 0))
     stop("DRomics cannot be used with negative values of doses.")
   design <- table(dose, dnn = "")
